
DECLARE

   TYPE aat_id IS TABLE OF source_table.object_id%TYPE
      INDEX BY PLS_INTEGER;
   TYPE aat_name IS TABLE OF source_table.object_name%TYPE
      INDEX BY PLS_INTEGER;
   TYPE aat_type IS TABLE OF source_table.object_type%TYPE
      INDEX BY PLS_INTEGER;

   aa_ids   aat_id;
   aa_names aat_name;
   aa_types aat_type;

   CURSOR c_source IS
      SELECT object_id
      ,      object_name
      ,      object_type
      FROM   source_table;

   n PLS_INTEGER := 0;

BEGIN

   OPEN c_source;
   LOOP
      FETCH c_source BULK COLLECT INTO aa_ids, aa_names, aa_types LIMIT 1000;

      --[some processing]--

      FORALL i IN 1 .. aa_ids.COUNT
         MERGE
            INTO  target_table tgt
            USING (
                   SELECT aa_ids(i) AS object_id
                   ,      aa_names(i) AS object_name
                   ,      aa_types(i) AS object_type
                   FROM   dual
                  ) src
            ON   ( src.object_id = tgt.object_id )
         WHEN MATCHED 
         THEN
            UPDATE
            SET   tgt.object_name = src.object_name
            ,     tgt.object_type = src.object_type
         WHEN NOT MATCHED 
         THEN
            INSERT ( tgt.object_id
                   , tgt.object_name
                   , tgt.object_type )
            VALUES ( src.object_id
                   , src.object_name
                   , src.object_type );

      n := n + SQL%ROWCOUNT;

      EXIT WHEN c_source%NOTFOUND;
   END LOOP;
   CLOSE c_source;

   DBMS_OUTPUT.PUT_LINE(n || ' rows merged.');
END;
/

ROLLBACK;
